//
//  ResistanceInputController.m
//  ResistanceInput
//
//  Created by Phidgets on 2015-07-29.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "ResistanceInputController.h"

@implementation ResistanceInputController

- (void)windowWillClose:(NSNotification*)aNotification {
    if(ch)
    {
        //Safely close example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetResistanceInput_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetResistanceInput_create(&ch);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = PhidgetResistanceInput_setOnResistanceChangeHandler(ch, gotResistanceChange, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}

#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}

void gotResistanceChange(PhidgetResistanceInputHandle phid, void *context, double resistance) {
        [(__bridge id)context performSelectorOnMainThread:@selector(onResistanceChangeHandler:)
                                               withObject:[NSNumber numberWithDouble:resistance]
                                            waitUntilDone:NO];
}

#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    uint32 dataInterval, minDataInterval, maxDataInterval;
    double resistanceChangeTrigger, minResistanceChangeTrigger,maxResistanceChangeTrigger;
    Phidget_RTDWireSetup wireSetup;
    
    //Get information from channel which will allow us to configure the GUI properly
    PhidgetResistanceInput_getMaxDataInterval(ch, &maxDataInterval);
    PhidgetResistanceInput_getMinDataInterval(ch, &minDataInterval);
    PhidgetResistanceInput_getDataInterval(ch, &dataInterval);
    PhidgetResistanceInput_getMaxResistanceChangeTrigger(ch, &maxResistanceChangeTrigger);
    PhidgetResistanceInput_getMinResistanceChangeTrigger(ch, &minResistanceChangeTrigger);
    PhidgetResistanceInput_getResistanceChangeTrigger(ch, &resistanceChangeTrigger);
    PhidgetResistanceInput_getRTDWireSetup(ch, &wireSetup);
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [dataBox setHidden:NO];
    [settingsBox setHidden:NO];
    
    //Configure data interval slider with information read from channel
    [maxDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",maxDataInterval]];
    [minDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",minDataInterval]];
    [dataIntervalSlider setMaxValue:(double)maxDataInterval];
    [dataIntervalSlider setMinValue:(double)minDataInterval];
    [dataIntervalSlider setIntValue:dataInterval];
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", dataInterval]];
    
    //Configure trigger slider with information read from channel
    [maxChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.1f Ω",maxResistanceChangeTrigger]];
    [minChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.1f Ω",minResistanceChangeTrigger]];
    [changeTriggerSlider setMaxValue:maxResistanceChangeTrigger];
    [changeTriggerSlider setMinValue:minResistanceChangeTrigger];
    [changeTriggerSlider setDoubleValue:resistanceChangeTrigger];
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.1f Ω", resistanceChangeTrigger]];
    
    //Configure rtd combo with information read from slider
    [rtdWireSetupCombo insertItemWithTitle:@"2 Wire" atIndex:0];
    [rtdWireSetupCombo insertItemWithTitle:@"3 Wire" atIndex:1];
    [rtdWireSetupCombo insertItemWithTitle:@"4 Wire" atIndex:2];
    
    if(wireSetup == RTD_WIRE_SETUP_2WIRE){
        [rtdWireSetupCombo selectItemAtIndex:0];
    }
    else if(wireSetup == RTD_WIRE_SETUP_3WIRE){
        [rtdWireSetupCombo selectItemAtIndex:1];
    }
    else if(wireSetup == RTD_WIRE_SETUP_4WIRE){
        [rtdWireSetupCombo selectItemAtIndex:2];
    }
    
    //Adjusting view height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}


- (void)onDetachHandler{
    //Taking items out of view
    [settingsBox setHidden:YES];
    [dataBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]
    	attributes:@{NSForegroundColorAttributeName: NSColor.controlTextColor}];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)outputLastError {
	const char *errorString;
	char *errorDetailString = NULL;
    size_t errorDetailStringLen = 0;
    PhidgetReturnCode lastErr;
	
    if (!Phidget_getLastError(&lastErr, &errorString, NULL, &errorDetailStringLen)) {
		errorDetailString = malloc(errorDetailStringLen);
		Phidget_getLastError(&lastErr, &errorString, errorDetailString, &errorDetailStringLen);
    	[self outputError:errorDetailString];
    	free(errorDetailString);
    }
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}
- (void)onResistanceChangeHandler:(NSNumber *)resistance{
    [resistanceText setStringValue:[NSString stringWithFormat:@"%0.3f Ω", resistance.doubleValue]];
}

#pragma mark GUI controls
- (IBAction)setChangeTrigger:(id)sender{
    PhidgetReturnCode result;
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.1f Ω", [changeTriggerSlider doubleValue]]];
    result = PhidgetResistanceInput_setResistanceChangeTrigger(ch, [changeTriggerSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
}

- (IBAction)setDataInterval:(id)sender{
    PhidgetReturnCode result;
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", [dataIntervalSlider intValue]]];
    result = PhidgetResistanceInput_setDataInterval(ch, [dataIntervalSlider intValue]);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
}
- (IBAction)setRTDWireSetup:(id)sender{
    PhidgetReturnCode result;
    if([[rtdWireSetupCombo titleOfSelectedItem]  isEqual: @"2 Wire"])
        result = PhidgetResistanceInput_setRTDWireSetup(ch, RTD_WIRE_SETUP_2WIRE);
    else if([[rtdWireSetupCombo titleOfSelectedItem]  isEqual: @"3 Wire"])
        result = PhidgetResistanceInput_setRTDWireSetup(ch, RTD_WIRE_SETUP_3WIRE);
    else if([[rtdWireSetupCombo titleOfSelectedItem]  isEqual: @"4 Wire"])
        result = PhidgetResistanceInput_setRTDWireSetup(ch, RTD_WIRE_SETUP_4WIRE);
	else
		result = EPHIDGET_UNEXPECTED;
    
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
}

@end
